/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is NewsFox.
 *
 * The Initial Developer of the Original Code is
 * Andy Frank <andy@andyfrank.com>.
 * Portions created by the Initial Developer are Copyright (C) 2005-2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Andrey Gromyko <andrey@gromyko.name>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

////////////////////////////////////////////////////////////////
// Global
////////////////////////////////////////////////////////////////

var checkInProgress = false;
var cancelCheck = false;
var refreshingAllFeeds = false;

////////////////////////////////////////////////////////////////
// Check feeds
////////////////////////////////////////////////////////////////

function doCheckFeeds()
{
  // Bail out if check already in progress
  if (checkInProgress) return;
  if (model.size() > 0) 
  {
    checkInProgress = true;
    var elem = document.getElementById("busyAnimation");
    elem.style.visibility = "visible";
    precheckFeed(0);
  }
}

function doCancelCheckFeeds()
{
  if (checkInProgress) cancelCheck = true;
}

function precheckFeed(index)
{
  if( index >= model.size() )
  {
    if( checkInProgress )
      cleanAfterCheck();
    return;
  }

  var elem = document.getElementById("busyTextNumbers");
  elem.value = (index+1) + " / " + model.size();
    
  var feed = model.get(index);
  feed.error = ERROR_OK;

  if( refreshingAllFeeds && !feed.autoCheck ) // then we skip
  {
    precheckFeed(index+1);
  }
  else
  {
    var xmlhttp = new XMLHttpRequest();    
    xmlhttp.open("get", feed.url);
    xmlhttp.setRequestHeader("User-Agent", "Mozilla/5.0 NewsFox/0.3.4");
    xmlhttp.overrideMimeType("application/xml");
    // TODO: do error handling
    // xmlhttp.onerror = foo;
    xmlhttp.onload = function() { checkFeed(index, xmlhttp.responseXML); }
    xmlhttp.send(null);
  }
}

function checkFeed(index, xml)
{
  try
  {
    var feed = model.get(index);    
    if (!feed.loaded)
    {
      loadFeed(feed);
      feed.loaded = true;
    }
    
    var parser = new Parser(xml);
    if (parser.title != null) 
      feed.defaultName = encodeString(parser.title);
    if (parser.link != null)
      feed.homepage = encodeUrl(parser.link);
    var oldestEntryDate = new Date();
                
    for (var i=0; i<parser.items.length; i++)
    {
      var item = parser.items[i];
      if (("" != item.pubDate) && (oldestEntryDate > item.pubDate)) 
        oldestEntryDate = item.pubDate;

      if (doesArticleExist(feed, item.link)) continue;
      
      var art = new Article();
      art.title = encodeString(item.title);
      art.link = item.link;
      art.date = item.pubDate;
      art.body = encodeString(item.content);
      art.read = false;
      art.flag = false;
      // AG: added category
      art.category = encodeString(item.category);
      
      if (art.date == "") art.date = new Date();
      if (art.title == "") art.title = formatDate(art.date);      
          
      feed.add(art);
    }

    feed.sort();
    feed.sortCategories();
    
    // AG: delete old not flagged entries if needed
    if (feed.deleteOld)
    {
      var feedSize = feed.size();
      for (var i=0; i<feedSize; i++)
      {
        var art = feed.get(i);
        if ((art.date < oldestEntryDate) && !feed.isFlagged(art))
        {
          feed.remove(i--);
          feedSize--;
        }
      }
    }
    
    saveFeed(feed);
  }    
  catch (err) 
  { 
    feed.error = err;
    feed.error += "<p><a href='" + feed.url + "'>" + feed.url + "</a>";    
  }
  
  // Update FeedTree
  var tree = document.getElementById("newsfox.feedTree");
  // by AG tree.treeBoxObject.invalidateRow(index);
  var curSelection = tree.currentIndex;
  tree.view = new FeedTreeModel(); // AG: to handle categories
  tree.view.selection.select(curSelection);
  
  if (!cancelCheck && index < model.size()-1)
  {
    // Check next feed
    precheckFeed(index+1);
  }
  else
  {
    // We're done!
    cleanAfterCheck();
  }
}

function cleanAfterCheck()
{
  var elem = document.getElementById("busyAnimation");
  elem.style.visibility = "hidden";
  checkInProgress = false;
  cancelCheck = false;
  refreshingAllFeeds = false;
  saveFeedModel();
  feedSelected();    
}

////////////////////////////////////////////////////////////////
// Util
////////////////////////////////////////////////////////////////

/**
 * Encode unicode characters to XML-friendly format.
 */
function encodeString(s) 
{
	if( !s ) return "";
  for (var i=0; i<s.length; i++)
  {
    var code = s.charCodeAt(i);
    if (code < 32 || code > 126)
    { 
      // Replacement character
      var replace = "&#" + code + ";";
      // Ignore newline characters and tabs      
      // TODO - should we ignore everything < 32 ?
      if (code == 9 || code == 10) replace = " ";
      s = s.substring(0, i) + replace + s.substring(i+1);
    }
  }
  return s;                                              
}

/**
 * Encode url problem charaters.
 */
function encodeUrl(s)
{
  s = s.replace(new RegExp('&','gi'), '&amp;');
  return s;
}

/**
 * Return true if this article already exists.  
 * We assume that every article has a unique link.
 */
function doesArticleExist(feed, link)
{
  for (var i=0; i<feed.size(); i++)
    if (feed.get(i).link == link) 
      return true;
  return false;
}

